/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng;

/**

	 SpagoBI - The Business Intelligence Free Platform

	 Copyright (C) 2005 Engineering Ingegneria Informatica S.p.A.

	 This library is free software; you can redistribute it and/or
	 modify it under the terms of the GNU Lesser General Public
	 License as published by the Free Software Foundation; either
	 version 2.1 of the License, or (at your option) any later version.

	 This library is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	 Lesser General Public License for more details.

	 You should have received a copy of the GNU Lesser General Public
	 License along with this library; if not, write to the Free Software
	 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/


/**
 * @author Daniela Tura
 *  
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Paint;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.DialShape;
import org.jfree.chart.plot.MeterInterval;
import org.jfree.chart.plot.MeterPlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.Range;
import org.jfree.data.general.DefaultValueDataset;
import org.jfree.ui.HorizontalAlignment;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.VerticalAlignment;

import net.sf.jasperreports.engine.JRScriptletException;


public class ReportUtilities  {


	private Connection dbConnection;
	private PreparedStatement stmt = null;
	private ResultSet rs = null;
	private String inArea="";
	private String inProject="";
	private String inMethod="";
	private String inCustomer="";
	




	public ReportUtilities(Connection dbConnection) {
		super();
		this.dbConnection = dbConnection;
	}

	public String buildPeriod(String pYear, String pMonth) {
		String queryPeriod = "";
		if (!pYear.equalsIgnoreCase("0")){
			queryPeriod += " and dt.year = " + pYear;
		}else{
			//queryPeriod += " and dt.year = " + String.valueOf((GregorianCalendar.getInstance()).get(Calendar.YEAR));
		}

		if (!pMonth.equalsIgnoreCase("0")){
			queryPeriod += " and dt.month = " + pMonth;
		}else{
			//queryPeriod += " and dt.month = " + String.valueOf((GregorianCalendar.getInstance()).get(Calendar.MONTH)+1);
		}
		return queryPeriod;
	}

	public String buildProfile(String pArea, String pProject, String tableArea, String tableProject) {
		String queryProfile = "";
		
		if ((pArea != null) &&  (!pArea.equalsIgnoreCase("-1"))){
			pArea = pArea.replace("'", "");
			String[] arrArea = pArea.split(",");
			String pArea2 = "";
			for (int i = 0; i < arrArea.length; i++) {
				pArea2 += "'" + arrArea[i] + "'";
				if (i < arrArea.length - 1)
					pArea2 += ",";
			}
			inArea=pArea2;
			queryProfile += " and "+tableArea+".id_organization IN (" + pArea2 + ")";
		}
		if ((pProject != null) &&  (!pProject.equalsIgnoreCase("-1"))){
			pProject = pProject.replace("'", "");
			String[] arrProject = pProject.split(",");
			String pProject2 = "";
			for (int i = 0; i < arrProject.length; i++) {
				pProject2 += "'" + arrProject[i] + "'";
				if (i < arrProject.length - 1)
					pProject2 += ",";
			}
			inProject=pProject2;
			queryProfile += " and "+tableProject+".id_project IN (" + pProject2 + ")";
		}
		return queryProfile;
	}


	public String buildCustomerMethod(String pCustomer, String pMethod, String tableCustomer, String tableMethod) {
		String queryProfile = "";
		if ((pCustomer != null) &&  (!pCustomer.equalsIgnoreCase("-1"))){
			pCustomer = pCustomer.replace("'", "");
			String[] arrCustomer = pCustomer.split(",");
			String pCustomer2 = "";
			for (int i = 0; i < arrCustomer.length; i++) {
				pCustomer2 += "'" + arrCustomer[i] + "'";
				if (i < arrCustomer.length - 1)
					pCustomer2 += ",";
			}
			inCustomer=pCustomer2;
			queryProfile += " and "+tableCustomer+".id_customer IN (" + pCustomer2 + ")";
		}
		if ((pMethod != null) &&  (!pMethod.equalsIgnoreCase("-1"))){
			pMethod = pMethod.replace("'", "");
			String[] arrMethod = pMethod.split(",");
			String pMethod2 = "";
			for (int i = 0; i < arrMethod.length; i++) {
				pMethod2 += "'" + arrMethod[i] + "'";
				if (i < arrMethod.length - 1)
					pMethod2 += ",";
			}
			inMethod=pMethod2;
			queryProfile += " and "+tableMethod+".id_methodology IN (" + pMethod2 + ")";
		}
		return queryProfile;
	}	    
	
	public String buildSeverity(String pSeverity, String tableName){
		String querySeverity="";
		if(pSeverity!=null){
			querySeverity+=" and "+tableName+".name ='"+pSeverity+"'";
		}
		return querySeverity;
	}


	public int calculateThreshold(int pThresholdType, int pIdUser, int pIdKPI)
	throws JRScriptletException {

		int defaultScale = 0;
		int selectedScale = 0;

		try {
			// select default scale (from kpi)
			String query = "select id_scale from dt_kpi where id_kpi = ?";
			stmt = dbConnection.prepareStatement(query);
			stmt.setInt(1, pIdKPI);
			stmt.execute();
			rs = stmt.executeQuery();
			rs.next();
			defaultScale = rs.getInt("id_scale");

			switch (pThresholdType) {
			case 0:
				// from kpi
				selectedScale = defaultScale;
				break;
			case 1:
				// from user-kpi
				query = "select id_scale from dt_user_kpi where id_user = ? and id_kpi = ?";
				stmt = dbConnection.prepareStatement(query);
				stmt.setInt(1, pIdUser);
				stmt.setInt(2, pIdKPI);
				stmt.execute();
				rs = stmt.executeQuery();
				if (rs.next()) {
					selectedScale = rs.getInt("id_scale");
				} else {
					selectedScale = defaultScale;
				}
				break;
				/*
				 * not managed yet case 2: // from domain-kpi query = "select
				 * id_scale from dt_domain_kpi where id_domain = ? and id_kpi = ?";
				 * stmt = dbConnection.prepareStatement(query);
				 * stmt.setInt(1,pIdDomain); stmt.setInt(2,pIdKPI); break;
				 */
			default:
				selectedScale = defaultScale;
			}
		} catch (Exception e) {
			throw new JRScriptletException(e);
		}
		return selectedScale;
	}

	public String getKPITitle(int pIdKPI) throws JRScriptletException {
		try {
			String query = "select name  from dt_kpi where id_kpi = ?";
			stmt = dbConnection.prepareStatement(query);
			stmt.setInt(1, pIdKPI);
			stmt.execute();
			rs = stmt.executeQuery();
			rs.next();
			return rs.getString("name");

		} catch (Exception e) {
			throw new JRScriptletException(e);
		}
	}

	public ArrayList<Pair> getSubtitles(String parArea, String parProject, String parCustomer, String parMethod) throws SQLException{
		String areaName="";
		String projectName="";
		String customerName="";
		String methodName="";
		String query="";

		Pair area=null;
		Pair project=null;
		Pair customer=null;
		Pair method=null;
		
		if(parArea.equalsIgnoreCase("-1"))areaName="All Organizations";
		else {
			query="SELECT name FROM spago4q.dt_organization WHERE id_organization IN ("+inArea+")";
			System.out.println("asub area "+query);
			stmt=dbConnection.prepareStatement(query);
			stmt.execute();
			rs = stmt.executeQuery();
			while (rs.next()) {
				areaName+=" ";
				areaName+=rs.getString("name");
				areaName+=";";
			}
			}
		area=new Pair("Area",areaName);
		
		if(parProject.equalsIgnoreCase("-1"))projectName="All Projects";
		else {
			query="SELECT name FROM spago4q.dt_project WHERE id_project IN ("+inProject+")";
			System.out.println("asub project "+query);
			stmt=dbConnection.prepareStatement(query);
			stmt.execute();
			rs = stmt.executeQuery();
			while (rs.next()) {
				projectName+=" ";
				projectName += rs.getString("name");
			projectName+=";";
			}
		}
		project=new Pair("Project",projectName);

		if(parCustomer.equalsIgnoreCase("-1"))customerName="All Customers";
		else {
			query="SELECT name FROM spago4q.dt_customer WHERE id_customer IN ("+inCustomer+")";
			stmt=dbConnection.prepareStatement(query);
			stmt.execute();
			rs = stmt.executeQuery();
			while(rs.next()) {
				customerName+=" ";
				customerName += rs.getString("name");
			customerName+=";";
			}
		}
		customer=new Pair("Customer",customerName);

		if(parMethod.equalsIgnoreCase("-1"))methodName="All Methodologies";
		else {
			query="SELECT name FROM spago4q.dt_methodology WHERE id_methodology IN ("+inMethod+")";
			stmt=dbConnection.prepareStatement(query);
			stmt.execute();
			rs = stmt.executeQuery();
			while(rs.next()) {
				methodName+=" ";
				methodName += rs.getString("name");
			methodName+=";";
			}
		}
		method=new Pair("Method",methodName);

		ArrayList<Pair> lista=new ArrayList<Pair>();
		lista.add(area);
		lista.add(project);
		lista.add(method);
		lista.add(customer);

		return lista;

	}


	public JFreeChart createMeterChart(BigDecimal pValue, String pTitle,
			int pIdThreshold, ArrayList<Pair> subs) throws JRScriptletException {
		JFreeChart chart = null;
		MeterPlot plot = null;

		try {
			// set value
			System.out.println(pValue);

			plot = new MeterPlot(new DefaultValueDataset(pValue));

			double min_value =0;
			double max_value=100;
			double min_range=0;
			double max_range=0;
			// set intervals
			ResultSet rs = this.extractScale(pIdThreshold);
			boolean first = true;
			while (rs.next()) {
				if (first) {
					first = false;
					min_range = rs.getDouble("min_value");
				}
				max_range = rs.getDouble("max_value");
				min_value = rs.getDouble("min_value");
				max_value = rs.getDouble("max_value");

				String label="";
				if(rs.getString("label")!=null)
					label=rs.getString("label");

				String colour = "#FFFFFF";
				if(rs.getString("colour")!=null)
					colour=rs.getString("colour");

				Range currentRange = new Range(min_value, max_value);
				String currentLabel = label;
				Color currentColour = Color.decode(colour);

				MeterInterval normal = new MeterInterval(currentLabel,
						currentRange, (Paint) currentColour, new BasicStroke(
								2.0f), (Paint) currentColour);
				plot.addInterval(normal);
			}

			// set range
			Range range = new Range(min_range, max_range);
			plot.setRange(range);

			// set properties
			plot.setDrawBorder(false);
			plot.setDialBackgroundPaint((Paint) Color.white);
			plot.setDialOutlinePaint((Paint) Color.white);
			plot.setNeedlePaint((Paint) Color.black);
			plot.setValuePaint((Paint) Color.black);
			plot.setUnits("");
			plot.setMeterAngle(180);
			plot.setTickLabelsVisible(true);
			plot.setDialShape(DialShape.CHORD);


			// create chart
			chart = new JFreeChart(pTitle, JFreeChart.DEFAULT_TITLE_FONT, plot,true);
			//chart=new JFreeChart(plot);
			Color lightGray=new Color(240,240,240);
			chart.getLegend().setBorder(BlockBorder.NONE);
			chart.getLegend().setBackgroundPaint((Paint)lightGray);
			chart.setBackgroundPaint((Paint)lightGray);

			String att="";
			String value="";
			TextTitle t;

			//Set subtitles
			for (int i=0;i<subs.size();i++) {
				att=subs.get(i).getAttName();
				value=subs.get(i).getValue();
				t=new TextTitle(att+": "+value);
				t.setFont(new Font("subtitle",0,8));
				t.setPosition(RectangleEdge.BOTTOM);
				t.setHorizontalAlignment(HorizontalAlignment.LEFT);
				t.setVerticalAlignment(VerticalAlignment.CENTER);
				chart.addSubtitle(t); 
				
			}

			return chart;
		} catch (Exception e) {
			throw new JRScriptletException(e);
		}
	}

	private ResultSet extractScale(int pIdThreshold)
	throws JRScriptletException {
		try {
			String query = "select min_value,max_value,label,colour from dt_scale_value"
				+ " where id_scale = ?" + " order by id_scale_value asc";
			stmt = dbConnection.prepareStatement(query);
			stmt.setInt(1, pIdThreshold);
			stmt.execute();
			rs = stmt.executeQuery();
			return rs;

		} catch (Exception e) {
			throw new JRScriptletException(e);
		}
	}
	
	public void fakeReports(String pArea, String pProject, String pCustomer, String pMethod){
		if ((pArea != null) &&  (!pArea.equalsIgnoreCase("-1"))){
			pArea = pArea.replace("'", "");
			String[] arrArea = pArea.split(",");
			String pArea2 = "";
			for (int i = 0; i < arrArea.length; i++) {
				pArea2 += "'" + arrArea[i] + "'";
				if (i < arrArea.length - 1)
					pArea2 += ",";
			}
			inArea=pArea2;
		}
		if ((pProject != null) &&  (!pProject.equalsIgnoreCase("-1"))){
			pProject = pProject.replace("'", "");
			String[] arrProject = pProject.split(",");
			String pProject2 = "";
			for (int i = 0; i < arrProject.length; i++) {
				pProject2 += "'" + arrProject[i] + "'";
				if (i < arrProject.length - 1)
					pProject2 += ",";
			}
			inProject=pProject2;
		}
		if ((pCustomer != null) &&  (!pCustomer.equalsIgnoreCase("-1"))){
			pCustomer = pCustomer.replace("'", "");
			String[] arrCustomer = pCustomer.split(",");
			String pCustomer2 = "";
			for (int i = 0; i < arrCustomer.length; i++) {
				pCustomer2 += "'" + arrCustomer[i] + "'";
				if (i < arrCustomer.length - 1)
					pCustomer2 += ",";
			}
			inCustomer=pCustomer2;
		}
		if ((pMethod != null) &&  (!pMethod.equalsIgnoreCase("-1"))){
			pMethod = pMethod.replace("'", "");
			String[] arrMethod = pMethod.split(",");
			String pMethod2 = "";
			for (int i = 0; i < arrMethod.length; i++) {
				pMethod2 += "'" + arrMethod[i] + "'";
				if (i < arrMethod.length - 1)
					pMethod2 += ",";
			}
			inMethod=pMethod2;
		}
		
	}
	
}




